/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            TemplateVertex.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEVERTEX_H
#define TEMPLATEVERTEX_H

#include "MeshTypes.h"

#include <vector>

namespace X3DTK {
namespace Mesh {

template<class A, class B, class C> class TemplateEdge;
template<class A, class B, class C> class TemplateFace;

/// Template class providing a Vertex class for the Mesh structure. This class can only
/// been constructed by a TemplateMesh.

template<class VData, class EData, class FData>
class TemplateVertex
{
  template<class A, class B, class C, class D> friend class TemplateMesh;
public:
  /// Gets the index.
  inline unsigned int getIndex() const {return _index;};
  /// Gets the rounding edges.
  inline const MTemplateEdge<EData, FData, VData> &getEdges() const {return _edges;};
  /// Gets the rounding faces.
  inline const MTemplateFace<FData, VData, EData> &getFaces() const {return _faces;};
  /// Gets the data.
  inline VData &data() {return _data;};
  /// Gets the data.
  inline const VData &data() const {return _data;};
  
private:
  const unsigned int _index;
  MTemplateEdge<EData, FData, VData> _edges;
  MTemplateFace<FData, VData, EData> _faces;
  VData _data;

  /// Constructor.
  TemplateVertex(unsigned int i)
  : _index(i) {};
  /// Adds an edge.
  void addEdge(TemplateEdge<EData, FData, VData> *E) {_edges.back() = E; _edges.push_back((TemplateEdge<EData, FData, VData> *)(0x1));};
  /// Adds a face.
  void addFace(TemplateFace<FData, VData, EData> *F) {_faces.back() = F; _faces.push_back((TemplateFace<FData, VData, EData> *)(0x1));};
};

}
}

#endif
