#include "DefaultInterpolationCreator.h"
#include <ColorInterpolator.h>
#include <CoordinateInterpolator.h>
#include <CoordinateInterpolator2D.h>
#include <NormalInterpolator.h>
#include <OrientationInterpolator.h>
#include <PositionInterpolator.h>
#include <PositionInterpolator2D.h>
#include <ScalarInterpolator.h>

namespace X3DTK {
namespace X3D {

DefaultInterpolationCreator::DefaultInterpolationCreator()
{
  defineNewCreationFunction<DefaultInterpolationCreator>("ColorInterpolator",        &DefaultInterpolationCreator::createColorInterpolator);
  defineNewCreationFunction<DefaultInterpolationCreator>("CoordinateInterpolator",   &DefaultInterpolationCreator::createCoordinateInterpolator);
  defineNewCreationFunction<DefaultInterpolationCreator>("CoordinateInterpolator2D", &DefaultInterpolationCreator::createCoordinateInterpolator2D);
  defineNewCreationFunction<DefaultInterpolationCreator>("NormalInterpolator",       &DefaultInterpolationCreator::createNormalInterpolator);
  defineNewCreationFunction<DefaultInterpolationCreator>("OrientationInterpolator",  &DefaultInterpolationCreator::createOrientationInterpolator);
  defineNewCreationFunction<DefaultInterpolationCreator>("PositionInterpolator",     &DefaultInterpolationCreator::createPositionInterpolator);
  defineNewCreationFunction<DefaultInterpolationCreator>("PositionInterpolator2D",   &DefaultInterpolationCreator::createPositionInterpolator2D);
  defineNewCreationFunction<DefaultInterpolationCreator>("ScalarInterpolator",       &DefaultInterpolationCreator::createScalarInterpolator);
}

DefaultInterpolationCreator::~DefaultInterpolationCreator()
{
}

SFNode DefaultInterpolationCreator::createColorInterpolator() const
{
  return new ColorInterpolator();
}

SFNode DefaultInterpolationCreator::createCoordinateInterpolator() const
{
  return new CoordinateInterpolator();
}

SFNode DefaultInterpolationCreator::createCoordinateInterpolator2D() const
{
  return new CoordinateInterpolator2D();
}

SFNode DefaultInterpolationCreator::createNormalInterpolator() const
{
  return new NormalInterpolator();
}

SFNode DefaultInterpolationCreator::createOrientationInterpolator() const
{
  return new OrientationInterpolator();
}

SFNode DefaultInterpolationCreator::createPositionInterpolator() const
{
  return new PositionInterpolator();
}

SFNode DefaultInterpolationCreator::createPositionInterpolator2D() const
{
  return new PositionInterpolator2D();
}

SFNode DefaultInterpolationCreator::createScalarInterpolator() const
{
  return new ScalarInterpolator();
}

}
}
