#include "FileValidatorGeometry3DVisitor.h"
#include "IndexedFaceSet.h"
#include "Coordinate.h"
#include "Normal.h"
#include "Color.h"
#include "ColorRGBA.h"
#include "TextureCoordinate.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

FileValidatorGeometry3DVisitor::FileValidatorGeometry3DVisitor()
: Geometry3DVisitor()
{
  globalVariables = GVManager::getInstanceOf<FileValidatorGlobalVariables>();
  
  // Enter functions.
  defineNewEnterFunction<FileValidatorGeometry3DVisitor, IndexedFaceSet>(&FileValidatorGeometry3DVisitor::enterIndexedFaceSet);
}

FileValidatorGeometry3DVisitor::~FileValidatorGeometry3DVisitor()
{
}

void FileValidatorGeometry3DVisitor::enterIndexedFaceSet(IndexedFaceSet *I) const
{
  Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
  if (CoordNode == NULL)
    return;
   
  int pointArraySize = 0;
  int normalArraySize = 0;
  int colorArraySize = 0;
  int texCoordArraySize = 0;
  
  X3D::Coordinate *CoordinateNode = dynamic_cast<X3D::Coordinate *>(I->getCoord());
  if (CoordinateNode != 0)
    pointArraySize = CoordNode->getPoint().size();
  

  X3D::Normal *NormalNode = dynamic_cast<X3D::Normal *>(I->getNormal());
  if (NormalNode != 0)
    normalArraySize = NormalNode->getVector().size();
  
  if (I->getColor() != 0)
  {
    if (I->getColor()->getTypeName() == "ColorRGBA")
      colorArraySize = dynamic_cast<X3D::ColorRGBA *>(I->getColor())->getColor().size();
    else  
      colorArraySize = dynamic_cast<X3D::Color *>(I->getColor())->getColor().size();
  }
  
  X3D::TextureCoordinate *TexCoordNode = dynamic_cast<X3D::TextureCoordinate *>(I->getTexCoord());
  if (TexCoordNode != 0)
    texCoordArraySize = TexCoordNode->getPoint().size();
  
  const MFInt32 &coordIndex = I->getCoordIndex();
  const MFInt32 &colorIndex = I->getColorIndex();
  const MFInt32 &normalIndex = I->getNormalIndex();
  const MFInt32 &texCoordIndex = I->getTexCoordIndex();

  bool hasPoint = ((pointArraySize != 0) || (coordIndex.size() > 1));
  bool hasNormal = ((normalArraySize != 0) || (normalIndex.size() > 1));
  bool hasColor = ((colorArraySize != 0) || (colorIndex.size() > 1));
  bool hasTexCoord = ((texCoordArraySize != 0) || (texCoordIndex.size() > 1));
  
  SFBool normalPerVertex = I->getNormalPerVertex();
  SFBool colorPerVertex = I->getColorPerVertex();
  
  MFInt32::const_iterator itCoordIndex = coordIndex.begin();
  MFInt32::const_iterator itColorIndex;
  MFInt32::const_iterator itNormalIndex;
  MFInt32::const_iterator itTexCoordIndex;
  
  if (colorIndex.empty())
    itColorIndex = coordIndex.begin();
  else
    itColorIndex = colorIndex.begin();  
  
  if (normalIndex.empty())
    itNormalIndex = coordIndex.begin();
  else
    itNormalIndex = normalIndex.begin();  
  
  if (texCoordIndex.empty())
    itTexCoordIndex = coordIndex.begin();
  else
    itTexCoordIndex = texCoordIndex.begin();
    
  while (itCoordIndex != coordIndex.end())
  {
    while ((*itCoordIndex != -1) && (itCoordIndex != coordIndex.end()))
    {
      if ((hasPoint) && (*itCoordIndex >= pointArraySize))
      {
        cout << "point " << *itCoordIndex << " doesn't exist!" << endl;
        globalVariables->setValidity(false);
      }

      if ((hasNormal) && (*itNormalIndex >= normalArraySize))
      {
        cout << "normal " << *itNormalIndex << " doesn't exist!" << endl;
        globalVariables->setValidity(false);
      }

      if ((hasColor) && (*itColorIndex >= colorArraySize))
      {
        cout << "color " << *itColorIndex << " doesn't exist!" << endl;
        globalVariables->setValidity(false);
      }

      if ((hasTexCoord) && (*itTexCoordIndex >= texCoordArraySize))
      {
        cout << "texCoord " << *itTexCoordIndex << " doesn't exist!" << endl;
        globalVariables->setValidity(false);
      }
        
      ++itCoordIndex;
      if (colorPerVertex)
        ++itColorIndex;     
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itTexCoordIndex;
    }
    
    ++itCoordIndex;
    if (!colorPerVertex)
      ++itColorIndex;
    ++itNormalIndex;
    ++itTexCoordIndex;
  }
}

}
}
