#include "NodeVisitingProxy.h"
#include "X3DComponentVisitor.h"
#include "AbstractVisitor.h"
#include "Type.h"

using namespace X3DTK;
using namespace std;

NodeVisitingProxy::NodeVisitingProxy()
{
  Type::addNodeProxy(this);
  setComponentVisitor(new AbstractVisitor());
}

NodeVisitingProxy::~NodeVisitingProxy()
{
  if (autoDelete)
    for (list<X3DComponentVisitor *>::iterator it = componentList_.begin(); it != componentList_.end(); ++it)
    {
      (*it)->removeOneProxy(); 
      if ((*it)->getProxyNumber() == 0)
        delete *it;
    }    

  Type::removeNodeProxy(this);
}

void NodeVisitingProxy::setComponentVisitor(X3DComponentVisitor *component)
{
  //replace the equivalent component.
  X3DComponentVisitor *replacedComponent = 0;
  bool replaced = false;
  for (list<X3DComponentVisitor *>::iterator it = componentList_.begin(); it != componentList_.end(); ++it)
  {
    if ((*it)->getName() == component->getName())
    {
      replacedComponent = *it;
      *it = component; 
      replacedComponent->removeOneProxy();
      component->addOneProxy();      
      replaced = true;
    }  
  }
  
  if (replacedComponent == component)
    return;
    
  if (!replaced)
  {
    componentList_.push_back(component);
    component->addOneProxy();
  }  
  
  //remove replaced component.
  if ((autoDelete) && (replacedComponent != 0) && (replacedComponent->getProxyNumber() == 0))
    delete replacedComponent;
    
  //iterating the types.
  for (unsigned int id = 0; id < visitingArray_.size(); ++id)
  {
    /// If the visitingArray_[id] == 0, then it means that the node of id id is not met.
    if (visitingArray_[id] != 0)
    {
      visitingArray_[id]->setEnterFunction(getEnterFunctionOf(Type::getTypeOfId(id)));
      visitingArray_[id]->setWalkOnFunction(getWalkOnFunctionOf(Type::getTypeOfId(id)));
      visitingArray_[id]->setLeaveFunction(getLeaveFunctionOf(Type::getTypeOfId(id)));
    }  
  }  
}

void NodeVisitingProxy::reset()
{
  //remove components
  if (autoDelete)
    for (list<X3DComponentVisitor *>::iterator it = componentList_.begin(); it != componentList_.end(); ++it)
    {
      (*it)->removeOneProxy();  
      if ((*it)->getProxyNumber() == 0)
        delete *it;
    }    
           
  componentList_.clear();
}

void NodeVisitingProxy::addType(const Type *type)
{
  if ((int)visitingArray_.size() <= type->getId())
    visitingArray_.resize(type->getId() + 1);
  
  visitingArray_[type->getId()] = new VisitingFunctions(getEnterFunctionOf(type), getWalkOnFunctionOf(type), getLeaveFunctionOf(type));
}

EnterFunction *NodeVisitingProxy::getEnterFunctionOf(const Type *type) const
{
  Type *t = (Type *)type;
  
  //searching the enter function.
  EnterFunction *EF = 0;
  while (t != 0)
  {
    for (list<X3DComponentVisitor *>::const_iterator it = componentList_.begin(); it != componentList_.end(); ++it)
    {
      EF = (*it)->getEnterFunctionOf(t);
      if (EF != 0)
        break;
    }
    if (EF != 0)
      break;
    
    t = t->getParent();
  }
/*  if (t != 0)
    cout << type->getName() << " " << type->getComponent()->getName() << " => Visiting Function = " << t->getName() << " " << t->getComponent()->getName() << endl;
  else
    cout << "NodeVisitingProxy::getEnterFunctionOf : no function found!" << endl;
  */
  return EF;
}

WalkOnFunction *NodeVisitingProxy::getWalkOnFunctionOf(const Type *type) const  
{
  Type *t = (Type *)type;
  
  //searching the walkOn function.
  WalkOnFunction *WF = 0;
  while (t != 0)
  {
    for (list<X3DComponentVisitor *>::const_iterator it = componentList_.begin(); it != componentList_.end(); ++it)
    {
      WF = (*it)->getWalkOnFunctionOf(t);
      if (WF != 0)
        break;
    }
    if (WF != 0)
      break;
    
    t = t->getParent();
  }

  return WF;
}  

LeaveFunction *NodeVisitingProxy::getLeaveFunctionOf(const Type *type) const
{
  Type *t = (Type *)type;

  //searching the leave function.
  LeaveFunction *LF = 0;
  while (t != 0)
  {
    for (list<X3DComponentVisitor *>::const_iterator it = componentList_.begin(); it != componentList_.end(); ++it)
    {
      LF = (*it)->getLeaveFunctionOf(t);
      if (LF != 0)
        break;
    }
    if (LF != 0)
      break;
    
    t = t->getParent();
  }
  
  return LF;
}

NodeVisitingProxy *X3DTK::joinNodeVisitingProxies(NodeVisitingProxy *N0, NodeVisitingProxy *N1)
{
  NodeVisitingProxy *N = new NodeVisitingProxy();
  
  for (list<X3DComponentVisitor *>::const_iterator it = N0->componentList_.begin(); it != N0->componentList_.end(); ++it)
    N->setComponentVisitor(*it);
  
  for (list<X3DComponentVisitor *>::const_iterator it = N1->componentList_.begin(); it != N1->componentList_.end(); ++it)
    N->setComponentVisitor(*it);
  
  return N;  
}
