#include "Type.h"
#include "X3DNodeProxy.h"
#include "Component.h"

#include <iostream>
#include <typeinfo>

using namespace X3DTK;
using namespace std;

int Type::count = 0;
MFType Type::_typeList = MFType();  
MFNodeProxy Type::_proxyList = MFNodeProxy();
bool Type::_functionDefinition = false;

void Type::setParent(const Type *parent)
{
  _parent = (Type *)parent;
}

void Type::addOneReference()
{
  ++_refCount;
}

void Type::addOneReferenceRecursive()
{
  ++_refCount;
  if (_parent != 0)
    _parent->addOneReferenceRecursive();
}

void Type::removeOneReference(Type *type)
{
  --type->_refCount;
  
  //removes the instance of the parents.
  if (type->_parent != 0)
    removeOneReference(type->_parent);
  
  SFString nodeName = type->_component->getName() + "_" + type->_name;
    
  if (type->_refCount == 0)  
  {
    //remove in the child list of the parent.
    
    if (!_functionDefinition)
    {
      //remove in the list.      
      _typeList.erase(nodeName);
    }
    delete type;
  }
}

Type *Type::getTypeOfName(const SFString &component, const SFString &name)
{
  SFString nodeName = component + "_" + name;
  MFType::const_iterator it = _typeList.find(nodeName); 
  if (it != _typeList.end())
    return (*it).second;
    
  //the type doesn't exist.
  return 0;  
}

Type *Type::getTypeOfId(int id)
{
  // iterate the Types
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
    if ((*it).second->getId() == id)
      return (*it).second;
      
  return 0;      
}
void Type::beginFunctionDefinition()
{
  _functionDefinition = true;
}

void Type::endFunctionDefinition()
{
  _functionDefinition = false;
}

void Type::recomputeIds()
{
}

void Type::printDerivationTree()
{
  cout << "Derivation tree:" << endl;
  SFString tab("");
  //finding the nodes which have no parent.
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
  {
    if ((*it).second->_parent == 0)
    {
      (*it).second->printDerivationTree(tab);
    } 
  }    
  cout << "Derivation tree finished" << endl;
}

Type::Type(const SFString &name)
: _name(name)
{
  //cout << "new Type " << name << endl;
}

Type::~Type()
{
  //cout << "destr Type = " << _name << endl;
  Component::removeOneReference(_component);
  _childList.clear();
}

void Type::defineTypeName(Type **type, const SFString &name, const SFString &component)
{
  SFString tcomponent = component;
  if (tcomponent == "")
    tcomponent = (*type)->_component->getName();
     
  Type *t = getTypeOfName(tcomponent, name);
  
  if (t == 0)
  {
    t = new Type(name);
    t->_id = count;
    t->_refCount = 0;
    t->_parent = *type;  
     
    //component update.
    t->_component = Component::getComponent(tcomponent);
    if (t->_component == 0)
      t->_component = new Component(tcomponent);

    //adding new name.
    SFString nodeName = tcomponent + "_" + name;
       
    if (*type != 0)    
      (*type)->_childList[nodeName] = t;
   
    if (!_functionDefinition)
    {
      ++count;
      _typeList[nodeName] = t;

      //advertising proxies
      for (MFNodeProxy::iterator itProxy = _proxyList.begin(); itProxy != _proxyList.end(); ++itProxy) 
        (*itProxy)->addType(t);
    }
       
    //cout << "def tag Type = " << name << endl;
    //component referenced by a new Type.
    t->_component->addOneReference();  
  }
  
  //the type t is referenced by a new Node.
  t->addOneReference();
  *type = t;
}

void Type::addNodeProxy(X3DNodeProxy *proxy)
{
  //updating proxy list
  _proxyList.push_back(proxy);
  
  //adding all types
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
    proxy->addType((*it).second);
}

void Type::removeNodeProxy(X3DNodeProxy *proxy)
{
  _proxyList.remove(proxy);
}

void Type::printDerivationTree(SFString tab) const
{
  cout << tab << _name << ": " << _refCount << ", " << _component->getName() << ": " << _component->_refCount << endl;
  tab += "  ";  

  for (MFType::const_iterator it = _childList.begin(); it != _childList.end(); ++it)
    (*it).second->printDerivationTree(tab);
}
