/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            NameDict.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef NAMEDICT_H
#define NAMEDICT_H

#include "X3DTypes.h"

#include <map>
#include <list>

namespace X3DTK {

/// Class providing a dictionary of list of names.

class NameDict
{
public:
  /// Constructor.
  NameDict();
  /// Destructor.
  ~NameDict();
  
  /// Inserts the couple (name, N).
  void insert(const SFString &name, const SFAbstractNode &N);
  /// Removes the couple of which the node is N.
  void remove(SFAbstractNode N);
  /// Sets the set of DEF non ambiguous, it means that it renames the common DEF. 
  /// It creates new DEF when there are more than one node referenced by the same DEF.
  /// The new name is "DEF"_i i>=0.
  void makeDEFNonAmbiguous();
  
  /// Gets the node corresponding to the name name. If the list contains more than 
  /// One element, the last element inserted is returned.
  SFAbstractNode getNodeOfName(const SFString &name) const;
  /// Gets the name corresponding to the node N.
  SFString getNameOfNode(const SFAbstractNode &N) const;
  
  ///Prints the content of the dictionary.
  void printDict() const;
  
private:
  std::map<SFString, std::list<SFAbstractNode>, ltSFString> _nameDict; 
}; 

}
 
#endif
