/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3DComponentCreator.h                           //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DCOMPONENTCREATOR_H
#define X3DCOMPONENTCREATOR_H

#include "X3DComponent.h"
#include "CreationFunction.h"

namespace X3DTK {

/// Abstract class defining the concept of component creator.

class X3DComponentCreator : public X3DComponent
{
public:
  /// Constructor.
  X3DComponentCreator();
  /// Destructor.
  virtual ~X3DComponentCreator() = 0;
  
  /// Gets the creation of name.
  CreationFunction *getCreationFunctionOf(const SFString &name) const;
  /// Returns the list of functions
  inline CreationDict getCreationDict() const {return creationDict_;};
  
  /// Returns TRUE if the component contains the type name.
  bool contains(const SFString &name) const;
  
protected:  
  /// Define a new creation function.
  template<class T> void defineNewCreationFunction(const SFString &name, X3D::SFNode (T::*ptrF)() const)
  {
    CreationFunction *CF = new CreationFunction(reinterpret_cast<ptrToCreationFunction>(ptrF), this);
    creationDict_.insert(std::pair<SFString, CreationFunction *>(name, CF));
  };
  
//private:
  CreationDict creationDict_;
};

}

#endif
