#include "X3DComponentWalker.h"
#include "Type.h"
#include "NodeWalkingProxy.h"
#include "NodeVisitingProxy.h"

using namespace X3DTK;
using namespace std;

X3DComponentWalker::X3DComponentWalker()
: nodeWalkingProxy(0), nodeVisitingProxy(0)
{
}

void X3DComponentWalker::setNodeWalkingProxy(const NodeWalkingProxy *nodeWalkingProxy)
{
  this->nodeWalkingProxy = (NodeWalkingProxy *)nodeWalkingProxy; 
}

void X3DComponentWalker::setNodeVisitingProxy(const NodeVisitingProxy *nodeVisitingProxy)
{
  this->nodeVisitingProxy = (NodeVisitingProxy *)nodeVisitingProxy; 
}

X3DComponentWalker::~X3DComponentWalker()
{
  //delete the creation functions
  for (WalkingDict::iterator it = walkingDict_.begin(); it != walkingDict_.end(); ++it)
    delete (*it).second; 
}

void X3DComponentWalker::setActiveAllFunctions()
{
  for (WalkingDict::const_iterator it = walkingDict_.begin(); it != walkingDict_.end(); ++it)
    (*it).second->setActive();
}

void X3DComponentWalker::setInactiveAllFunctions()
{
  for (WalkingDict::const_iterator it = walkingDict_.begin(); it != walkingDict_.end(); ++it)
    if ((*it).first != "X3DNode")
      (*it).second->setInactive();
}

WalkingFunction *X3DComponentWalker::getWalkingFunctionOf(const Type *type) const
{
  SFString nodeName = type->getComponent()->getName() + "_" + type->getName();
  WalkingDict::const_iterator it = walkingDict_.find(nodeName);
  if ((it != walkingDict_.end()) && ((*it).second->isActive()))
      return (*it).second;
      
  //not found  
  return 0;  
}
