#include "DefaultDFSNURBSWalker.h"
#include "NodeWalkingProxy.h"
#include "NodeVisitingProxy.h"
#include "NurbsSurface.h"
#include "TrimmedSurface.h"

namespace X3DTK {
namespace X3D {

DefaultDFSNURBSWalker::DefaultDFSNURBSWalker()
: DFSNURBSWalker()
{
  defineNewWalkingFunction<DefaultDFSNURBSWalker, NurbsSurface>(&DefaultDFSNURBSWalker::walkNurbsSurface);
  defineNewWalkingFunction<DefaultDFSNURBSWalker, TrimmedSurface>(&DefaultDFSNURBSWalker::walkTrimmedSurface);
}

DefaultDFSNURBSWalker::~DefaultDFSNURBSWalker()
{
}

void DefaultDFSNURBSWalker::walkNurbsSurface(NurbsSurface *N) const
{
  nodeVisitingProxy->enter(N);

  SFNode C = N->getTexCoord();
  if ((C != 0) && (nodeVisitingProxy->walkOn(N, C)))
    nodeWalkingProxy->walk(C);

  nodeVisitingProxy->leave(N);
}

void DefaultDFSNURBSWalker::walkTrimmedSurface(TrimmedSurface *T) const
{
  nodeVisitingProxy->enter(T);

  MFNode trimmingContour = T->getTrimmingContour();
  for (MFNode::iterator it = trimmingContour.begin(); it != trimmingContour.end(); ++it)
    if ((*it != 0) && (nodeVisitingProxy->walkOn(T, *it)))
      nodeWalkingProxy->walk(*it);

  SFNode C = T->getSurface();
  if ((C != 0) && (nodeVisitingProxy->walkOn(T, C)))
    nodeWalkingProxy->walk(C);

  nodeVisitingProxy->leave(T);
}

}
}
