#include "DefaultDFSRenderingWalker.h"
#include "NodeWalkingProxy.h"
#include "NodeVisitingProxy.h"
#include "X3DComposedGeometryNode.h"
#include "IndexedLineSet.h"
#include "PointSet.h"

namespace X3DTK {
namespace X3D {

DefaultDFSRenderingWalker::DefaultDFSRenderingWalker()
: DFSRenderingWalker()
{
  defineNewWalkingFunction<DefaultDFSRenderingWalker, X3DComposedGeometryNode>(&DefaultDFSRenderingWalker::walkX3DComposedGeometryNode);
  defineNewWalkingFunction<DefaultDFSRenderingWalker, IndexedLineSet>(&DefaultDFSRenderingWalker::walkIndexedLineSet);
  defineNewWalkingFunction<DefaultDFSRenderingWalker, PointSet>(&DefaultDFSRenderingWalker::walkPointSet);
}

DefaultDFSRenderingWalker::~DefaultDFSRenderingWalker()
{
}

void DefaultDFSRenderingWalker::walkX3DComposedGeometryNode(X3DComposedGeometryNode *N) const
{
  nodeVisitingProxy->enter(N);
  
  SFNode C;
  C = N->getColor();
  if ((C != 0) && (nodeVisitingProxy->walkOn(N, C)))
    nodeWalkingProxy->walk(C);
    
  C = N->getCoord();
  if ((C != 0) && (nodeVisitingProxy->walkOn(N, C)))
    nodeWalkingProxy->walk(C);
    
  C = N->getNormal();
  if ((C != 0) && (nodeVisitingProxy->walkOn(N, C)))
    nodeWalkingProxy->walk(C);
    
  C = N->getTexCoord();
  if ((C != 0) && (nodeVisitingProxy->walkOn(N, C)))
    nodeWalkingProxy->walk(C);    
  
  nodeVisitingProxy->leave(N);
}

void DefaultDFSRenderingWalker::walkIndexedLineSet(IndexedLineSet *I) const
{
  nodeVisitingProxy->enter(I);
  
  SFNode C;
  C = I->getColor();
  if ((C != 0) && (nodeVisitingProxy->walkOn(I, C)))
    nodeWalkingProxy->walk(C);
    
  C = I->getCoord();
  if ((C != 0) && (nodeVisitingProxy->walkOn(I, C)))
    nodeWalkingProxy->walk(C);
    
  nodeVisitingProxy->leave(I);
}

void DefaultDFSRenderingWalker::walkPointSet(PointSet *P) const
{
  nodeVisitingProxy->enter(P);
  
  SFNode C;
  C = P->getColor();
  if ((C != 0) && (nodeVisitingProxy->walkOn(P, C)))
    nodeWalkingProxy->walk(C);
    
  C = P->getCoord();
  if ((C != 0) && (nodeVisitingProxy->walkOn(P, C)))
    nodeWalkingProxy->walk(C);
    
  nodeVisitingProxy->leave(P);
}

}
}
