#include "GLStaticGroup.h"
#include "StaticGroup.h"

#include <algorithm>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace GL {

StaticGroup::StaticGroup()
: X3DChildNode()
{  
  defineTypeName("StaticGroup", "GLGrouping");
}

StaticGroup::StaticGroup(const StaticGroup &G)
: X3DChildNode(G), _children(G._children)
{
}

SFNode StaticGroup::clone() const
{
  return new StaticGroup(*this);
}

StaticGroup::~StaticGroup()
{  
}

void StaticGroup::setChildren(const MFNode &children)
{
  _children = children;
}

bool StaticGroup::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    _children.push_back(static_cast<SFNode>(N));
    N->addParent(this);
    return true;
  }
  
  cerr << "StaticGroup::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;    
}

bool StaticGroup::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    _children.push_back(static_cast<SFNode>(N));
    N->addParent(this);
    return true;
  }
  
  cerr << "StaticGroup::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;   
}

bool StaticGroup::removeChild(const SFAbstractNode &N)
{
  MFNode::iterator res = find(_children.begin(), _children.end(), static_cast<SFNode>(N));
  if (res != _children.end())
  {
    (*res)->removeParent(this);
    _children.erase(res);
    return true;
  }  
  return false;
}

}
}
