#include "Cylinder.h"

namespace X3DTK {
namespace X3D {

Cylinder::Cylinder()
: X3DGeometry3DNode(), _bottom(true), _radius(1.0f), _height(2.0f), _side(true), _top(true)  
{
  defineTypeName("Cylinder", "Geometry3D");
}

Cylinder::Cylinder(SFBool bottom, SFFloat radius, SFFloat height, SFBool side, SFBool top)
: X3DGeometry3DNode(), _bottom(bottom), _radius(radius), _height(height), _side(side), _top(top)
{
  defineTypeName("Cylinder", "Geometry3D");
}

Cylinder::Cylinder(const Cylinder &C)
: X3DGeometry3DNode(C), _bottom(C._bottom), _radius(C._radius), _height(C._height), _side(C._side), _top(C._top)
{
}

SFNode Cylinder::clone() const
{
  return new Cylinder(*this);
}

Cylinder::~Cylinder()
{
}

void Cylinder::setBottom(SFBool bottom)
{
  _bottom = bottom;
}

void Cylinder::setRadius(SFFloat radius)
{
  _radius = radius;
}

void Cylinder::setHeight(SFFloat height)
{
  _height = height;
}

void Cylinder::setSide(SFBool side)
{
  _side = side;
}

void Cylinder::setTop(SFBool top)
{
  _top = top;
}

void Cylinder::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("bottom");
  if (index != -1)
    _bottom = (element->getAttribute(index).upper() == "TRUE");
  
  index = element->getIndexAttribute("radius");
  if (index != -1)
    _radius = element->getAttribute(index).toFloat(); 

  index = element->getIndexAttribute("height");
  if (index != -1)
    _height = element->getAttribute(index).toFloat(); 
  
  index = element->getIndexAttribute("side");
  if (index != -1)
    _side = (element->getAttribute(index).upper() == "TRUE"); 
  
  index = element->getIndexAttribute("top");
  if (index != -1)
    _top = (element->getAttribute(index).upper() == "TRUE"); 
}

SFString Cylinder::write() const
{
  SFString attr;
  if (!_bottom)
    attr += " bottom=\"FALSE\"";
  if (_radius != 1.0f)  
    attr += " radius=\"" + toSFString(_radius) + "\"";
  if (_height != 2.0f)  
    attr += " height=\"" + toSFString(_height) + "\"";
  if (!_side)  
    attr += " side=\"FALSE\"";
  if (!_top)    
    attr += " top=\"FALSE\"";
      
  return attr;
}

}
}
