#include "StaticGroup.h"

#include <algorithm>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

StaticGroup::StaticGroup()
: X3DChildNode(), X3DBoundedObject()
{
  defineTypeName("StaticGroup", "Grouping");
}

StaticGroup::StaticGroup(const SFVec3f &bboxCenter, const SFVec3f &bboxSize)
: X3DChildNode(), X3DBoundedObject(bboxCenter, bboxSize)
{
  defineTypeName("StaticGroup", "Grouping");
}

StaticGroup::StaticGroup(const StaticGroup &G)
: X3DChildNode(G), X3DBoundedObject(G), _children(G._children)
{
}

SFNode StaticGroup::clone() const
{
  return new StaticGroup(*this);
}

StaticGroup::~StaticGroup()
{
}

bool StaticGroup::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    _children.push_back(static_cast<SFNode>(N));
    addLink(this, static_cast<SFNode>(N));
    return true;
  }
  
  cerr << "StaticGroup::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;    
}

bool StaticGroup::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DChildNode *>(N) != 0)
  {
    _children.push_back(static_cast<SFNode>(N));
    addLink(this, static_cast<SFNode>(N));
    return true;
  }
  
  cerr << "StaticGroup::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;    
}

bool StaticGroup::removeChild(const SFAbstractNode &N)
{
  MFNode::iterator res = find(_children.begin(), _children.end(), N);
  if (res != _children.end())
  {
    _children.erase(res);
    removeLink(this, static_cast<SFNode>(N));
    return true;
  }
      
  return false;
}

void StaticGroup::load(const X3DFileElement *element)
{
  X3DBoundedObject::load(element);
}

SFString StaticGroup::write() const
{
  return X3DBoundedObject::write();
}

}
}
