#include "PositionInterpolator.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

PositionInterpolator::PositionInterpolator()
  : X3DInterpolatorNode(),
    _keyValue(MFVec3f())
{
  defineTypeName("PositionInterpolator", "Interpolation");
}

PositionInterpolator::PositionInterpolator(const MFFloat &key,
                                           const MFVec3f &keyValue)
  : X3DInterpolatorNode(key),
    _keyValue(keyValue)
{
  defineTypeName("PositionInterpolator", "Interpolation");
}

PositionInterpolator::PositionInterpolator(const PositionInterpolator &N)
  : X3DInterpolatorNode(N),
    _keyValue(N._keyValue)
{
}

SFNode PositionInterpolator::clone() const
{
  return new PositionInterpolator(*this);
}

PositionInterpolator::~PositionInterpolator()
{
}

void PositionInterpolator::setKeyValue(const MFVec3f &keyValue)
{
  _keyValue = keyValue;
}

void PositionInterpolator::load(const X3DFileElement *element)
{
  X3DInterpolatorNode::load(element);

  int index;
  index = element->getIndexAttribute("keyValue");
  if (index != -1)
    _keyValue = MFVec3f(element->getAttribute(index));
}

SFString PositionInterpolator::write() const
{
  SFString attr;
  if (!_keyValue.empty())
    attr += " keyValue=\"" + toSFString(_keyValue) + "\"";

  return X3DInterpolatorNode::write() + attr;  
}

}
}
