/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3DInterpolatorNode.h                           //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef X3DINTERPOLATORNODE_H
#define X3DINTERPOLATORNODE_H

#include "X3DTypes.h"
#include "X3DChildNode.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/interpolation.html">
/// Interpolation component</a> defining an X3DInterpolatorNode
/// (<a href="../../doc/X3DSpecs/Part01/components/interpolation.html#X3DInterpolatorNode">
/// documentation</a>).

class X3DInterpolatorNode : public X3DChildNode
{
public:
  /// Constructor.
  X3DInterpolatorNode();
  /// Constructs an X3DInterpolatorNode from its attributes.
  X3DInterpolatorNode(const MFFloat &key);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DInterpolatorNode();

  /// Sets the key attribute.
  void setKey(const MFFloat &key);

  /// Gets the key attribute.
  inline const MFFloat &getKey() const {return _key;};

  /// Loads the attributes from a X3DFileElement.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.
  X3DInterpolatorNode(const X3DInterpolatorNode &N);

private:
  /// List of key times.
  MFFloat _key;
};

}
}

#endif
