#include "X3DLightNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

X3DLightNode::X3DLightNode()
  : X3DChildNode(),
    _ambientIntensity(0.0f),
    _color           (SFColor(1.0f,1.0f,1.0f)),
    _intensity       (1.0f),
    _on              (true)
{
  defineTypeName("X3DLightNode", "Lighting");
}

X3DLightNode::X3DLightNode(SFFloat       ambientIntensity,
                           const SFColor &color,
                           SFFloat       intensity,
                           SFBool        on)
  : X3DChildNode(),
    _ambientIntensity(ambientIntensity),
    _color           (color),
    _intensity       (intensity),
    _on              (on)
{
  defineTypeName("X3DLightNode", "Lighting");
}

X3DLightNode::X3DLightNode(const X3DLightNode &N)
  : X3DChildNode(N),
    _ambientIntensity(N._ambientIntensity),
    _color           (N._color),
    _intensity       (N._intensity),
    _on              (N._on)
{
}

SFNode X3DLightNode::clone() const
{
  return 0;
}

X3DLightNode::~X3DLightNode()
{
}

void X3DLightNode::setAmbientIntensity(SFFloat ambientIntensity)
{
  _ambientIntensity = ambientIntensity;
}

void X3DLightNode::setColor(const SFColor &color)
{
  _color = color;
}

void X3DLightNode::setIntensity(SFFloat intensity)
{
  _intensity = intensity;
}

void X3DLightNode::setOn(SFBool on)
{
  _on = on;
}

void X3DLightNode::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("ambientIntensity");
  if (index != -1)
    _ambientIntensity = element->getAttribute(index).toFloat();

  index = element->getIndexAttribute("color");
  if (index != -1)
    _color = SFColor(element->getAttribute(index));

  index = element->getIndexAttribute("intensity");
  if (index != -1)
    _intensity = element->getAttribute(index).toFloat();

  index = element->getIndexAttribute("on");
  if (index != -1)
    _on = (element->getAttribute(index).upper() == "TRUE");
}

SFString X3DLightNode::write() const
{
  SFString attr;
  if (_ambientIntensity != 0.0f)
    attr += " ambientIntensity=\"" + toSFString(_ambientIntensity) + "\"";
  if (_color != SFColor(1.0f,1.0f,1.0f))  
    attr += " color=\"" + toSFString(_color) + "\"";
  if (_intensity != 1.0f)  
    attr += " intensity=\"" + toSFString(_intensity) + "\"";
  if (!_on)  
    attr += " on=\"FALSE\"";
    
  return attr;  
}

}
}
