#include "NurbsGroup.h"
#include "NurbsSurface.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

NurbsGroup::NurbsGroup()
  : X3DGroupingNode(), X3DBoundedObject(),
    _tesselationScale(1.0f)
{
  defineTypeName("NurbsGroup", "NURBS");
}

NurbsGroup::NurbsGroup(const SFVec3f &bboxCenter,
                       const SFVec3f &bboxSize,
                       SFFloat tesselationScale)
  : X3DGroupingNode(), X3DBoundedObject(bboxCenter, bboxSize),
    _tesselationScale(tesselationScale)
{
  defineTypeName("NurbsGroup", "NURBS");
}

NurbsGroup::NurbsGroup(const NurbsGroup &N)
  : X3DGroupingNode(N), X3DBoundedObject(N),
    _tesselationScale(N._tesselationScale)
{
}

SFNode NurbsGroup::clone() const
{
  return new NurbsGroup(*this);
}

NurbsGroup::~NurbsGroup()
{
}

bool NurbsGroup::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<NurbsSurface *>(N) != 0)
    return X3DGroupingNode::addChild(N);
  
  cerr << "NurbsGroup::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;    
}

bool NurbsGroup::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<NurbsSurface *>(N) != 0)
    return X3DGroupingNode::setChild(N);
  
  cerr << "NurbsGroup::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;    
}

void NurbsGroup::load(const X3DFileElement *element)
{
  X3DBoundedObject::load(element);

  int index;
  index = element->getIndexAttribute("tesselationScale");
  if (index != -1)
    _tesselationScale = element->getAttribute(index).toFloat();
}

SFString NurbsGroup::write() const
{
  SFString attr;
  if (_tesselationScale != 1.0f)
    attr += " tesselationScale=\"" + toSFString(_tesselationScale) + "\"";

  return X3DBoundedObject::write() + attr;
}

}
}
