#include "GLBuilderShapeVisitor.h"
#include "Appearance.h"
#include "FillProperties.h"
#include "LineProperties.h"
#include "Material.h"
#include "Shape.h"
#include "GLAppearance.h"
#include "GLFillProperties.h"
#include "GLLineProperties.h"
#include "GLMaterial.h"
#include "GLShape.h"

namespace X3DTK {
namespace X3D {

GLBuilderShapeVisitor::GLBuilderShapeVisitor()
: ShapeVisitor()
{
  // Enter functions.
  defineNewEnterFunction<GLBuilderShapeVisitor, Appearance>(&GLBuilderShapeVisitor::enterAppearance);
  defineNewEnterFunction<GLBuilderShapeVisitor, FillProperties>(&GLBuilderShapeVisitor::enterFillProperties);
  defineNewEnterFunction<GLBuilderShapeVisitor, LineProperties>(&GLBuilderShapeVisitor::enterLineProperties);
  defineNewEnterFunction<GLBuilderShapeVisitor, Material>(&GLBuilderShapeVisitor::enterMaterial);
  defineNewEnterFunction<GLBuilderShapeVisitor, Shape>(&GLBuilderShapeVisitor::enterShape);

  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLBuilderGlobalVariables>();
}

GLBuilderShapeVisitor::~GLBuilderShapeVisitor()
{
}

void GLBuilderShapeVisitor::enterAppearance(Appearance *A) const
{
  GL::Appearance *GA = new GL::Appearance();
  GA->setX3DReference(A);
  globalVariables->getTop()->addChild(GA);
  globalVariables->pushNode(GA);
}

void GLBuilderShapeVisitor::enterFillProperties(FillProperties *F) const
{
  GL::FillProperties *GF = new GL::FillProperties();
  GF->setX3DReference(F);
  globalVariables->getTop()->addChild(GF);
  globalVariables->pushNode(GF);
}

void GLBuilderShapeVisitor::enterLineProperties(LineProperties *L) const
{
  GL::LineProperties *GL = new GL::LineProperties();
  GL->setX3DReference(L);
  globalVariables->getTop()->addChild(GL);
  globalVariables->pushNode(GL);
}

void GLBuilderShapeVisitor::enterMaterial(Material *M) const
{
  GL::Material *GM = new GL::Material();
  GM->setX3DReference(M);
  globalVariables->getTop()->addChild(GM);
  globalVariables->pushNode(GM);
}

void GLBuilderShapeVisitor::enterShape(Shape *S) const
{
  GL::Shape *GS = new GL::Shape();
  GS->setX3DReference(S);
  globalVariables->getTop()->addChild(GS);
  globalVariables->pushNode(GS);
}

}
}
