/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            TemplateMeshBuilder.h                           //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEMESHBUILDER_H
#define TEMPLATEMESHBUILDER_H

#include "MeshTypes.h"
#include "X3DOnePassProcessor.h"
#include "TemplateMeshBuilderGlobalVariables.h"
#include "DefaultDFS.h"
#include "TemplateMeshBuilderCoreVisitor.h"
#include "TemplateMeshBuilderGeometry3DVisitor.h"
#include "TemplateMeshBuilderGroupingVisitor.h"
#include "TemplateMeshBuilderShapeVisitor.h"
#include "TemplateMeshBuilderRenderingVisitor.h"
#include "Scene.h"
#include "MeshScene.h"
#include "MeshAlgoFunctions.h"

namespace X3DTK {
namespace X3D {

/// Processor defining a builder of template meshes.

template<class MData, class VData, class EData, class FData>
class TemplateMeshBuilder : public X3DOnePassProcessor
{     
public:
  /// Constructor. 
  TemplateMeshBuilder() 
  {
    globalVariables = GVManager::getInstanceOf<TemplateMeshBuilderGlobalVariables>();
    dfs = new DefaultDFS();
  
    dfs->setComponentVisitor(new TemplateMeshBuilderCoreVisitor());
    dfs->setComponentVisitor(new TemplateMeshBuilderGeometry3DVisitor<MData, VData, EData, FData>());
    dfs->setComponentVisitor(new TemplateMeshBuilderGroupingVisitor());
    dfs->setComponentVisitor(new TemplateMeshBuilderShapeVisitor());
    dfs->setComponentVisitor(new TemplateMeshBuilderRenderingVisitor());
  };
  /// Virtual destructor.
  virtual ~TemplateMeshBuilder() 
  {
    delete dfs;
  };
  /// Sets the component visitor.
  void setComponentVisitor(X3DComponentVisitor *component)
  {
    dfs->setComponentVisitor(component);
  };
    
  /// Builds the mesh from N.
  virtual Mesh::SFNode build(SFNode N)
  {
    std::cout << "building mesh" << std::endl;
    globalVariables->init();
    dfs->traverse(N);
    globalVariables->finish();
    
    return globalVariables->getRoot();
  };
  
  /// Builds the mesh scene from S.
  Mesh::Scene *build(Scene *S)
  {
    return static_cast<Mesh::Scene *>(build(static_cast<SFNode>(S)));
  };
  
private:
  TemplateMeshBuilderGlobalVariables *globalVariables;
  DFS *dfs; 
};

}
}

#endif
