#include "TemplateMeshBuilderGroupingVisitor.h"
#include "Group.h"
#include "StaticGroup.h"
#include "Transform.h"
#include "MeshTransform.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TemplateMeshBuilderGroupingVisitor::TemplateMeshBuilderGroupingVisitor()
: GroupingVisitor()
{
  // Enter functions.
  defineNewEnterFunction<TemplateMeshBuilderGroupingVisitor, Group>(&TemplateMeshBuilderGroupingVisitor::enterGroup);
  defineNewEnterFunction<TemplateMeshBuilderGroupingVisitor, StaticGroup>(&TemplateMeshBuilderGroupingVisitor::enterStaticGroup);
  defineNewEnterFunction<TemplateMeshBuilderGroupingVisitor, Transform>(&TemplateMeshBuilderGroupingVisitor::enterTransform);

  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<TemplateMeshBuilderGlobalVariables>();
}

TemplateMeshBuilderGroupingVisitor::~TemplateMeshBuilderGroupingVisitor()
{
}

void TemplateMeshBuilderGroupingVisitor::enterGroup(Group *G) const
{
  Mesh::Transform *MT = new Mesh::Transform();
  MT->setTransform(identity34());
  globalVariables->getTop()->addChild(MT);
  globalVariables->pushNode(MT);
}

void TemplateMeshBuilderGroupingVisitor::enterStaticGroup(StaticGroup *S) const
{
  Mesh::Transform *MT = new Mesh::Transform();
  MT->setTransform(identity34());  
  globalVariables->getTop()->addChild(MT);
  globalVariables->pushNode(MT); 
}

void TemplateMeshBuilderGroupingVisitor::enterTransform(Transform *T) const
{
  Mesh::Transform *MT = new Mesh::Transform();
  
  SFMatrix34f Tr = translation(T->getTranslation());
  SFMatrix34f C = translation(T->getCenter());
  SFVec3f v(T->getRotation().x, T->getRotation().y, T->getRotation().z);
  SFMatrix34f R = rotation(T->getRotation().angle, v);
  SFVec3f vs(T->getScaleOrientation().x, T->getScaleOrientation().y, T->getScaleOrientation().z);
  SFMatrix34f SR = rotation(T->getScaleOrientation().angle, vs);
  SFMatrix34f S = scale34(T->getScale().x, T->getScale().y, T->getScale().z);
  SFMatrix34f iSR = rotation(-T->getScaleOrientation().angle, vs);
  SFMatrix34f iC = translation(-1.0f*T->getCenter());
  SFMatrix34f transform = Tr*C*R*SR*S*iSR*iC;
    
  MT->setTransform(transform);
  globalVariables->getTop()->addChild(MT);
  globalVariables->pushNode(MT);
}

}
}
